local addonName, FSF = ...
local L = FSF.L

local FSFConfigPanel = FSF.frame
FSFConfigPanel.name = addonName
FSFConfigPanel:Hide()

--------------------------------------
-- new widget function
--------------------------------------
-- CheckButton
local function NewCheckButton(frame, label, tooltip, onClickFn)
	local cb = CreateFrame("CheckButton", addonName .. "ConfigCheckButton_" .. label, frame, "InterfaceOptionsCheckButtonTemplate")
	cb:SetScript("OnClick", function(self)
		PlaySound(self:GetChecked() and "igMainMenuOptionCheckBoxOn" or "igMainMenuOptionCheckBoxOff")
		onClickFn(self, self:GetChecked() and true or false) 
	end)
	cb.label = _G[cb:GetName() .. "Text"]
	cb.label:SetText(label)
	if tooltip then
		cb.tooltipText = label
		cb.tooltipRequirement = tooltip
	end
	
	return cb
end

-- Slider
local function NewSlider(frame, label, low, high, tooltip, OnValueChangedFn)
	local slider = CreateFrame("Slider", addonName .. "ConfigSlider_" .. label, frame, "OptionsSliderTemplate")
    slider:SetMinMaxValues(low, high)
    slider:SetValueStep(0.5)
	slider:SetObeyStepOnDrag(true)
    slider.text = _G[slider:GetName() .. "Text"]
    slider.low = _G[slider:GetName() .. "Low"]
    slider.high = _G[slider:GetName() .. "High"]

    slider.text:SetText(label)
    slider.low:SetText("")
    slider.high:SetText("")
    if tooltip then slider.tooltipText = tooltip end
	
	local vText = slider:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	vText:SetHeight(35)
    vText:SetPoint("LEFT", slider, "RIGHT", 5, 0)

    slider:SetScript("OnValueChanged", function(self, value)
		vText:SetText(value)
        OnValueChangedFn(self, value)
    end)
	
	return slider
end

-- List
local PADDING = 2
local buttons = {}

local function NewListButton(frame, label, onRemoveFn)
	-- button
	local b = CreateFrame("Button", nil, frame)
	b:SetHeight(17)
	b:SetScript("OnEnter", function(self) self.removeButton:Show() end)
	b:SetScript("OnLeave", function(self) if not self.removeButton:IsMouseOver() then self.removeButton:Hide() end end)
	
	-- remove button
	local removeButton = CreateFrame("Button", nil, b, "UIPanelCloseButton")
	removeButton:SetSize(17, 17)
	removeButton:SetPoint("RIGHT")
	removeButton:Hide()
	removeButton:SetScript("OnClick", onRemoveFn)
	removeButton:SetScript('OnEnter', function(self) b:LockHighlight() end)
	removeButton:SetScript('OnLeave', function(self) 
		if not b:IsMouseOver() then self:Hide() end 
		b:UnlockHighlight()
	end)
	b.removeButton = removeButton
	
	-- highlight
	local highlight = b:CreateTexture(nil, "BACKGROUND")
	highlight:SetTexture([[Interface\QuestFrame\UI-QuestLogTitleHighlight]])
	highlight:SetVertexColor(0.196, 0.388, 0.8)
	highlight:SetBlendMode("ADD")
	highlight:SetAllPoints(b)
	b:SetHighlightTexture(highlight)
	
	-- text
	local text = b:CreateFontString(nil, "ARTWORK")
	text:SetJustifyH("LEFT")
	text:SetAllPoints(b)
	b:SetFontString(text)
	b:SetNormalFontObject('GameFontNormal')
	b:SetHighlightFontObject('GameFontHighlight')
	
	b:SetText(label)
	return b
end

--------------------------------------
-- functions
--------------------------------------
local function CanFixFont(fontName)
	local exists = false
	for _, v in pairs(FSF.cfg.specificFonts) do
		if fontName == v then
			exists = true
		end
	end
	
	if pcall(function() FSF.getFontInstance(fontName):GetShadowOffset() end) and not exists then
		return true
	end
	return false
end

local function Remove(rBtn)
	-- print("remove -- " .. rBtn:GetParent():GetText())
	for k, v in pairs(FSF.cfg.specificFonts) do
		if rBtn:GetParent():GetText() == v then
			table.remove(FSF.cfg.specificFonts, k)
		end
	end
	
	local ht = rBtn:GetParent():GetHighlightTexture()
	ht:SetVertexColor(0.8, 0.1, 0.3)
	
	rBtn:SetScript("OnEnter", nil)
	rBtn:SetScript("OnLeave", nil)
	rBtn:GetParent():LockHighlight()
end

local function Add(frame, text, init)
	if init or CanFixFont(text) then
		buttons[#buttons+1] = NewListButton(frame, text, function(self)
			Remove(self)
		end)
		if #buttons == 1 then
			buttons[1]:SetPoint("TOPLEFT")
			buttons[1]:SetPoint("TOPRIGHT")
		else
			buttons[#buttons]:SetPoint("TOPLEFT", buttons[#buttons-1], "BOTTOMLEFT", 0, -PADDING)
			buttons[#buttons]:SetPoint("TOPRIGHT", buttons[#buttons-1], "BOTTOMRIGHT", 0, -PADDING)
		end
		if not init then table.insert(FSF.cfg.specificFonts, text) end
	end
end

local function Load(frame)
	for _, v in pairs(FSF.cfg.specificFonts) do
		Add(frame, v, true)
	end
end

--------------------------------------
-- FSFConfigPanel OnShow
--------------------------------------
FSFConfigPanel:SetScript("OnShow", function(frame)
	frame:SetScript("OnShow", nil)
	
	-- text
    local title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
    title:SetPoint("TOPLEFT", 16, -16)
    title:SetText(addonName)
	
	local desc1 = frame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
    desc1:SetHeight(35)
	desc1:SetJustifyH("LEFT")
    desc1:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -8)
    desc1:SetText(L.DESC1 .. " (" .. FSF.cfg.fixed .. " fixed)")
	
	local desc2 = frame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
    desc2:SetHeight(35)
	desc2:SetJustifyH("LEFT")
    desc2:SetPoint("TOPLEFT", title, "TOPLEFT", 0, -190)
    desc2:SetText(L.DESC2)
	
	-- check button	
	local fixAll = NewCheckButton(
		frame, L.FIX_ALL, L.FIX_ALL_DESC,
		function(self, value) 
			FSF.cfg.fixAll = value
		end)
	fixAll:SetChecked(FSF.cfg.fixAll)
	fixAll:SetPoint("TOPLEFT", title, "BOTTOMLEFT", -2, -116)
	
	local fixSpecific = NewCheckButton(
		frame, L.FIX_SPECIFIC, L.FIX_SPECIFIC_DESC,
		function(self, value)
			FSF.cfg.fixSpecific = value
		end)
	fixSpecific:SetChecked(FSF.cfg.fixSpecific)
	fixSpecific:SetPoint("TOPLEFT", title, "BOTTOMLEFT", -2, -146)
	
	-- slider
	local xslider = NewSlider(
		frame, L.X_OFFSET, -5, 5, nil,
		function(self, value) FSF.cfg.xoffset = value end)
	xslider:SetValue(FSF.cfg.xoffset)
	xslider:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -70)
	
	local yslider = NewSlider(
		frame, L.Y_OFFSET, -5, 5, nil,
		function(self, value) FSF.cfg.yoffset = value end)
	yslider:SetValue(FSF.cfg.yoffset)
	yslider:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 200, -70)
	
	-- button
	local fix = CreateFrame("Button", addonName .. "ConfigButton_Edit", frame, "UIPanelButtonTemplate")
	fix:SetText(L.FIX)
	fix:SetWidth(100)
	fix:SetHeight(24)
	fix:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -470)
	fix:SetScript("OnClick", function()
		FSF.fix()
		desc1:SetText(L.DESC1 .. " (" .. FSF.cfg.fixed .. " fixed)")
	end)
	fix.tooltipText = L.FIX_DESC
	
	local rescan = CreateFrame("Button", addonName .. "ConfigButton_Reset", frame, "UIPanelButtonTemplate")
	rescan:SetText(L.RESCAN)
	rescan:SetWidth(100)
	rescan:SetHeight(24)
	rescan:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 120, -470)
	rescan:SetScript("OnClick", function()
		FSF.rescan()
	end)
	rescan.tooltipText = L.RESCAN_DESC

	local reload = CreateFrame("Button", addonName .. "ConfigButton_Reload", frame, "UIPanelButtonTemplate")
	reload:SetText(L.RELOAD)
	reload:SetWidth(100)
	reload:SetHeight(24)
	reload:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 240, -470)
	reload:SetScript("OnClick", function()
		_G.ReloadUI()
	end)

	-- list
	local listScrollFrame = CreateFrame("ScrollFrame", addonName .. "ConfigScrollFrame", FSFConfigPanel, "UIPanelScrollFrameTemplate")
	local scrollChild = CreateFrame("Frame", addonName .. "ConfigScrollFrameChild", FSFConfigPanel)
	
	scrollChild:SetHeight(100)
	scrollChild:SetWidth(370)
	-- local tex = scrollChild:CreateTexture()
	-- tex:SetAllPoints(scrollChild)
	-- tex:SetColorTexture(0.7,0.7,0.7,0.2)
	
	listScrollFrame:SetScrollChild(scrollChild)
	listScrollFrame:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -235)
	listScrollFrame:SetPoint("BOTTOMRIGHT", title, "BOTTOMLEFT", 370, -455)
	listScrollFrame:SetBackdrop({
		bgFile = [[Interface\Buttons\WHITE8x8]], 
		-- edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]], 
		-- tile = true, tileSize = 16, edgeSize = 16,
		-- insets = { left = 4, right = 4, top = 4, bottom = 4 }
	})
	listScrollFrame:SetBackdropColor(0, 0, 0, 0.5)
	
	Load(scrollChild)	-- Load from SV
	
	-- edit box
	local editBox = CreateFrame("EditBox", addonName .. "ConfigEditBox", frame, "InputBoxTemplate")
	editBox:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 5, -205)
	editBox:SetPoint("BOTTOMRIGHT", title, "BOTTOMLEFT", 370, -225)
	editBox:SetScript('OnEnterPressed', function(self) Add(scrollChild, self:GetText()) end)
	editBox:SetScript('OnTextChanged', editBox_OnTextChanged)
	editBox:SetAutoFocus(false)
	
	-- local b = NewListButton(scrollChild, "test", function(self) print(self:GetParent():GetText()) end)
	-- b:SetPoint("TOPLEFT")
	-- b:SetPoint("TOPRIGHT")
	
	-- Supported addons
	-- local supported = CreateFrame("Frame", addonName .. "ConfigSupportedAddonsFrame", FSFConfigPanel)
	-- supported:SetPoint("TOPLEFT", listScrollFrame, "TOPRIGHT", 50, 0)
	-- supported:SetPoint("BOTTOMRIGHT", listScrollFrame, "BOTTOMRIGHT", 200, 0)
	-- supported:SetBackdrop({
		-- bgFile = [[Interface\Buttons\WHITE8x8]],
	-- })
	-- supported:SetBackdropColor(0, 0, 0, 0.5)
	
	-- local title_sa = supported:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	-- title_sa:SetJustifyH("CENTER")
    -- title_sa:SetPoint("TOP", supported, "TOP", 0, -8)
    -- title_sa:SetText(L.TITLE_SA)
end)

InterfaceOptions_AddCategory(FSFConfigPanel)