local _, FSF = ...
local L = FSF.L
--------------------------------------
-- Functions
--------------------------------------
local function GetFontInstance(s)
	local ret = "return _G." .. s
	-- for m in string.gmatch(s, "[^.]+") do
		-- ret =  ret .. "[\"" ..  m .. "\"]"
	-- end
	
	if not pcall(function() ret = loadstring(ret)() end) then ret = nil end
	return ret
end
FSF.getFontInstance = GetFontInstance

local function SetShadowOffset(font, xoffset, yoffset)
	if pcall(function() font:GetShadowOffset() end) then -- Attempt to access forbidden object from code tainted by an AddOn :(
		x, y = font:GetShadowOffset()
		if x ~= xoffset or y ~= yoffset then
			font:SetShadowOffset(xoffset, yoffset)
		end
		return true
	else
		return false
	end
end

local function FixFontInstance(rescan, fontInstances, xoffset, yoffset)
	if rescan then
		for name, obj in pairs(_G) do
			if type(obj) == "table" and SetShadowOffset(obj, xoffset, yoffset) then
				table.insert(fontInstances, name)
			end
		end
	else
		for _, name in pairs(fontInstances) do
			SetShadowOffset(GetFontInstance(name), xoffset, yoffset)
		end
	end
	return #fontInstances	-- fixed count
end

StaticPopupDialogs["FSF_RESCAN"] = {
	text = L.FSF_RESCAN,
	button1 = "Yes",
	button2 = "No",
	OnAccept = function()
		wipe(FontShadowFixConfig.fixedFonts)
		_G.ReloadUI()
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
}

local function Rescan()
	StaticPopup_Show("FSF_RESCAN")
end

local function Fix()
	local count = 0
	if FontShadowFixConfig.fixAll then
		if #FontShadowFixConfig.fixedFonts == 0 then
			-- print("fix all(rescan)")
			count = FixFontInstance(true, FontShadowFixConfig.fixedFonts, FontShadowFixConfig.xoffset, FontShadowFixConfig.yoffset) + count
		else
			-- print("fix all(scaned)")
			count = FixFontInstance(false, FontShadowFixConfig.fixedFonts, FontShadowFixConfig.xoffset, FontShadowFixConfig.yoffset) + count
		end
	end
	if FontShadowFixConfig.fixSpecific then
		-- print("fix specific")
		 count = FixFontInstance(false, FontShadowFixConfig.specificFonts, FontShadowFixConfig.xoffset, FontShadowFixConfig.yoffset) + count
	end
	FontShadowFixConfig.fixed = count
end

FSF.rescan = Rescan
FSF.fix = Fix
--------------------------------------
-- Event
--------------------------------------
local frame = CreateFrame("Frame", nil, InterfaceOptionsFramePanelContainer)
frame:RegisterEvent("ADDON_LOADED")
frame:RegisterEvent("PLAYER_LOGIN")
FSF.frame = frame

function frame:ADDON_LOADED(arg1)
	if arg1 == "FontShadowFix" then
		frame:UnregisterEvent("ADDON_LOADED")
		if type(FontShadowFixConfig) ~= "table" then FontShadowFixConfig = {} end
		if type(FontShadowFixConfig.fixAll) ~= "boolean" then FontShadowFixConfig.fixAll = true end
		if type(FontShadowFixConfig.fixSpecific) ~= "boolean" then FontShadowFixConfig.fixSpecific = false end
		if type(FontShadowFixConfig.fixed) ~= "number" then FontShadowFixConfig.fixed = 0 end
		if type(FontShadowFixConfig.xoffset) ~= "number" then FontShadowFixConfig.xoffset = 1 end
		if type(FontShadowFixConfig.yoffset) ~= "number" then FontShadowFixConfig.yoffset = -1 end
		if type(FontShadowFixConfig.fixedFonts) ~= "table" then FontShadowFixConfig.fixedFonts = {} end
		if type(FontShadowFixConfig.specificFonts) ~= "table" then FontShadowFixConfig.specificFonts = {} end
		
		FSF.cfg = FontShadowFixConfig
	end
end

function frame:PLAYER_LOGIN()
	Fix()
end

frame:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)

--------------------------------------
-- Slash
--------------------------------------
-- SLASH_FONTSHADOWFIX1 = "/fsf"
-- function SlashCmdList.FONTSHADOWFIX(msg)
-- end